import {
  ApolloClient,
  ApolloLink,
  InMemoryCache,
  createHttpLink
} from '@apollo/client';
import { MiddlewareConfig } from '../index';
import { HttpsProxyAgent } from 'https-proxy-agent';
import fetch from 'node-fetch';

const buildClient = (settings: MiddlewareConfig) => {
  const agent = new HttpsProxyAgent(process.env.PROXY);

  const httpLink = createHttpLink({
    uri: settings.odooGraphqlUrl,
    credentials: 'include',
    fetch: settings.fetch || fetch,
    fetchOptions: process.env.PROXY ? { ...settings.fetchOptions, agent } : settings.fetchOptions,
    headers: settings.headers
  });

  const afterwareLink = new ApolloLink((operation, forward) => {
    return forward(operation).map((response) => {
      const context = operation.getContext();
      const authHeader = context.response.headers.get('set-cookie');

      if (response.data) {
        response.data.cookie = authHeader;
      }

      return response;
    });
  });

  const apolloLink = ApolloLink.from([afterwareLink.concat(httpLink)]);

  return new ApolloClient({
    link: apolloLink,
    cache: new InMemoryCache(),
    ssrMode: true,
    credentials: 'include',
    defaultOptions: {
      query: {
        errorPolicy: 'all',
        fetchPolicy: 'no-cache'
      },
      mutate: {
        errorPolicy: 'all'
      }
    }
  });
};

export default buildClient;
